program DBCExport;

{$mode Delphi}{$H+}

uses
  {$IFDEF UNIX}{$IFDEF UseCThreads}
  cthreads,
  {$ENDIF}{$ENDIF}
  Classes, SysUtils, CustApp
  { you can add units after this },
  USqlDatabase, UDBC, Variants;

type
  { TDBCExport }
  TDBCExport = class(TCustomApplication)
  private
    procedure ParseParameters;
  protected
    procedure DoRun; override;
  public
    DBC: TDBC;
    Database: TSqlDatabase;
    DBCFileName: string;
    NewDBCFileName: string;
    ColumnTypeDefinition: string;
    SQLquery: string;
    DisplayData: Boolean;
    constructor Create(TheOwner: TComponent); override;
    destructor Destroy; override;
    procedure WriteHelp; virtual;
    procedure ShowDBC;
    procedure LoadDBC;
    procedure SaveDBC;
    procedure ReplaceText;
    function GetIDbyEntry(entry: integer; var DBRows: TDBRows): integer;
  end;

{ TDBCExport }

procedure TDBCExport.ParseParameters;
begin
  if HasOption('h', 'help') then begin
    WriteHelp;
    Halt;
  end;
  if HasOption('f', 'file') then begin
    DBCFileName := GetOptionValue('f', 'file');
    NewDBCFileName := Copy(DBCFileName, 1, Length(DBCFileName) -
      Length(ExtractFileExt(DBCFileName))) + '-2.dbc';
  end else begin
    WriteLn('Source DBC file not specified.');
    Halt;
  end;
  if HasOption('n', 'newfile') then begin
    NewDBCFileName := GetOptionValue('n', 'newfile');
  end;
  if HasOption('c', 'coltypedef') then begin
    ColumnTypeDefinition := GetOptionValue('c', 'coltypedef');
  end;
  if HasOption('d', 'display') then begin
    DisplayData := True;
  end;

  if HasOption('s', 'schema') then begin
    Database.Database := GetOptionValue('s', 'schema');
  end else Database.Database := 'wowpreklad';
  if HasOption('u', 'user') then begin
    Database.UserName := GetOptionValue('u', 'user');
  end else Database.UserName := 'root';
  if HasOption('p', 'password') then begin
    Database.Password := GetOptionValue('p', 'password');
  end else Database.Password := '';
  if HasOption('t', 'host') then begin
    Database.Hostname := GetOptionValue('t', 'host');
  end else Database.Hostname := 'localhost';
  if HasOption('q', 'sql_query') then begin
    SQLquery := GetOptionValue('q', 'sql_query');
  end;
end;

procedure TDBCExport.DoRun;
begin
  ParseParameters;

  LoadDBC;

  ShowDBC;

  ReplaceText;

  SaveDBC;

  // Stop program loop
  Terminate;
end;

constructor TDBCExport.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  StopOnException := True;
  Database := TSqlDatabase.Create;
end;

destructor TDBCExport.Destroy;
begin
  Database.Destroy;
  inherited Destroy;
end;

procedure TDBCExport.WriteHelp;
begin
  WriteLn('Usage: ', ExtractFileName(ExeName), ' [options]');
  WriteLn('  -h --help        Show this help');
  WriteLn('  -u --user        Database user name');
  WriteLn('  -p --password    Database password name');
  WriteLn('  -s --schema      Database schema name');
  WriteLn('  -t --host        Database host name');
  WriteLn('  -f --file        Source DBC file name');
  WriteLn('  -n --newfile     New created DBC file name');
  WriteLn('  -c --coltypedef  DBC column type definition string (u - uint32, s - string)');
  WriteLn('  -d --display     Display DBC data');
  WriteLn('  -q --sql_query  SQL select when entry is ID first column in dbc file and text0,text1... is texts columns');
  // SELECT T.entry as entry,T.Text as text0 FROM (SELECT * FROM gametips WHERE (Complete = 1) AND ((Language = 1)) AND (User IN (459,670,602,462,1,400,638,592,624,610,769,331,131,704,2,499,641,660,578,337,304,277,208,613,768,754,590,606,26,618,739,503,601,607,585,596,765,320,547,687)) AND VersionStart <= 9947 AND VersionEnd >= 9947) AS T GROUP BY T.entry
end;

procedure TDBCExport.ShowDBC;
var
  X, Y: Integer;
  Text: string;
begin
  with DBC do begin
      // Display DBC data
      if DisplayData then
      for Y := 0 to Length(Cells) - 1 do begin
        Text := '';
        for X := 0 to Length(Cells[Y]) - 1 do
         if VarIsStr(Cells[Y, X]) then Text := Text + Cells[Y, X] + ', '
          else Text := Text + IntToStr(Cells[Y, X]) + ', ';
        WriteLn(Text);
      end;
  end;
end;

procedure TDBCExport.LoadDBC;
var
  I: Integer;
begin
  DBC := TDBC.Create;
  with DBC do begin
    if FileExists(DBCFileName) then begin
      LoadFromFile(DBCFileName);

      // Load strings for string columns
      for I := 1 to Length(ColumnTypeDefinition) do
        if ColumnTypeDefinition[I] = 's' then
          LoadColumnStrings(I - 1);

    end else WriteLn('File ' + DBCFileName + ' not exists.');
  end;
end;

procedure TDBCExport.SaveDBC;
begin
  with DBC do begin
    SaveToFile(NewDBCFileName);
    Free;
  end;
end;


//function get data from database and replace text in dbc
procedure TDBCExport.ReplaceText;
var
  DBRows : TDbRows;
  Text: String;
  X, Y, I: Integer;
  IndexRowData: Integer;
begin
  with DBC do begin
    if (SQLquery <> '') then begin;
      Database.Connect;
      DBRows := Database.Query(SQLquery);


      for Y := 0 to Length(DBC.Cells) - 1 do begin
        if (Y = 129) then
               IndexRowData:= GetIDbyEntry(Cells[Y,0],DBRows);
        IndexRowData:= GetIDbyEntry(Cells[Y,0],DBRows);
        I := 0;
        if (IndexRowData <> -1) then begin
          for X := 0 to Length(ColumnTypeDefinition)-1 do begin
            if ColumnTypeDefinition[X+1] = 's' then begin
              Text:=DBRows.Data[IndexRowData].Values['text'+IntToStr(I)];
              SetString(Y,X,Text);
              I:=I+1;
            end;
          end;
          DBRows.Delete(IndexRowData);
        end;
      end;



    end;
  end;
end;

function TDBCExport.GetIDbyEntry(entry: integer; var DBRows: TDBRows): integer;
var
 i: integer;
 Count: integer;
begin
  try
     Count:=DBRows.Count;
     for i:=0 to DBRows.Count - 1 do begin
      if (StrToInt(DBRows.Data[i].Values['entry']) = entry) then begin
        Result := i;
        exit;
      end;
    end;
  except
  end;
  Result := -1;
end;

var
  Application: TDBCExport;

{$IFDEF WINDOWS}{$R DBCExport.rc}{$ENDIF}

{$R DBCExport.res}

begin
  Application := TDBCExport.Create(nil);
  Application.Run;
  Application.Free;
end.

